"""This is a test for implement async loading of weather. Not really working yet"""

import asyncio
import curses

async def load_weather():
    # your weather loading code here
    await asyncio.sleep(3)  # simulate some work
    return "New weather"


async def draw_interface(stdscr):
    # your curses interface drawing code here
    weather = ""
    weather = await load_weather()
    while True:
        stdscr.addstr(0, 0, weather)
        stdscr.refresh()
        key = stdscr.getkey()
        stdscr.refresh()


def main():
    stdscr = curses.initscr()
    # curses.cbreak()
    stdscr.keypad(True)
    stdscr.clear()

    loop = asyncio.get_event_loop()
    loop.create_task(draw_interface(stdscr))
    loop.run_forever()

    stdscr.keypad(False)
    curses.echo()
    curses.endwin()

if __name__ == '__main__':
    main()



"""Italian translations of the program interface"""

MSG_WELCOME_1 = "Benvenuto in Calcure"
MSG_WELCOME_2 = "Il tuo calendario da terminale e gestore di attività!"
MSG_WELCOME_3 = "File di dati e di configurazione creati al:"
MSG_WELCOME_4 = "Per ulteriore supporto, per contribuire e per avere informazioni aggiuntive, visitare:"
MSG_WELCOME_5 = "Premi ? per visualizzare tutte le scorciatoie da tastiera o qualunque altro tasto per continuare."

TITLE_KEYS_GENERAL = "SCORCIATOIE DA TASTIERA GENERALI"
TITLE_KEYS_CALENDAR = "SCORCIATOIE DA TASTIERA PER IL CALENDARIO"
TITLE_KEYS_JOURNAL  = "SCORCIATOIE DA TASTIERA PER IL DIARIO"

KEYS_GENERAL = {
        " Barra spaziatrice ": "Cambia fra calendario e diario",
        "   /   ": "Attiva la divisione dello schermo",
        "   *   ": "Attiva la privacy globale",
        "   ?   ": "Aiuto",
        "   q   ": "Esci",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Aggiungi un evento (ricorrente)",
        "  n,🠒  ": "Mese successivo (giorno)",
        "  p,🠐  ": "Mese precedente (giorno)",
        "  d,x  ": "Cancella un evento",
        "  e,c  ": "Modifica un evento",
        "   m   ": "Sposta un evento",
        "   v   ": "Alterna la visualizzazione giornaliera/mensile",
        "   g   ": "Vai ad un determinato giorno",
        "   h   ": "Imposta un evento come ad alta priorità",
        "   l   ": "Imposta un evento come a bassa priorità",
        "   .   ": "Imposta la privacy per un evento",
        "   C   ": "Importa eventi da calcurse",
        "   G   ": "Ritorna al mese corrente (giorno)",
        }

KEYS_TODO = {
        "  a(A) ": "Aggiungi una nuova (sotto)attività",
        "  h(H) ": "Imposta una fra (o tutte) le attività come ad alta priorità",
        "  l(L) ": "Imposta una fra (o tutte) le attività come a bassa priorità",
        "  v(V) ": "Imposta una fra (o tutte) le attività come completate",
        "  u(U) ": "Deseleziona una fra (o tutte) le attività",
        "  d(D) ": "Cancella una fra (o tutte) le attività (con tutte le sue sottoattività)",
        "  t(T) ": "Avvia/ferma (o cancella) il timer per una attività",
        "  e,c  ": "Modifica una attività",
        "   s   ": "Passa da attività a sottoattività (e viceversa)",
        "   .   ": "Imposta la privacy dell'attività",
        "  f(F) ": "Cambia (o rimuovi) la scadenza per una attività",
        "   m   ": "Sposta una attività",
        "  C(W) ": "Importa le attività da calcurse (taskwarrior)",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "È possibile usare i tasti di Vim (j, k, ZZ, ZQ)!"
MSG_INFO          = "Per maggiori informazioni, visitare:"
MSG_SITE          = "https://github.com/anufrievroman/calcure"
MSG_EXIT          = "Vuoi davvero uscire? (y/n) "

MSG_EVENT_HIGH    = "Imposta ad alta priorità l'evento con numero: "
MSG_EVENT_LOW     = "Imposta come a bassa priorità l'evento con numero: "
MSG_EVENT_RESET   = "Ripristina lo stato per l'evento con numero: "
MSG_EVENT_DEL     = "Cancella l'evento con numero: "
MSG_EVENT_REN     = "Rinomina l'evento con numero: "
MSG_NEW_TITLE     = "Inserisci il nuovo titolo: "
MSG_EVENT_MOVE    = "Sposta l'evento con numero: "
MSG_EVENT_MOVE_TO = "Postponi l'evento a: "
MSG_EVENT_DATE    = "Inserisci una data: "
MSG_EVENT_TITLE   = "Inserisci un titolo: "
MSG_EVENT_REP     = "Quante volte vuoi ripetere l'evento: "
MSG_EVENT_FR      = "Ripetere l'evento ogni giorno(d), settimana(w), mese(m) o anno(y)? "
MSG_EVENT_IMP     = "Vuoi importare un evento da Calcurse? (y/n)"
MSG_EVENT_PRIVACY = "Attiva la privacy dell'evento con numero: "
MSG_TM_ADD        = "Avvia/ferma il timer per l'attività con numero: "
MSG_TM_RESET      = "Rimuovi il timer per l'attività con numero: "
MSG_TS_HIGH       = "Imposta come ad alta priorità l'attività con numero: "
MSG_TS_LOW        = "Imposta come a bassa priorità l'attività con numero: "
MSG_TS_RES        = "Ripristina lo stato per l'attività con numero: "
MSG_TS_DONE       = "Segna come completata l'attività con numero: "
MSG_TS_DEL        = "Cancella l'attività con numero: "
MSG_TS_DEL_ALL    = "Vuoi davvero cancellare tutte le attività? (y/n)"
MSG_TS_MOVE       = "Sposta il task dal numero: "
MSG_TS_MOVE_TO    = "Sposta il task al numero: "
MSG_TS_EDIT       = "Modifica il numero dell'attività: "
MSG_TS_TOG        = "Attiva la sottoattività con numero: "
MSG_TS_SUB        = "Aggiungi una sottoattività all'attività con numero: "
MSG_TS_TITLE      = "Inserisci la sottoattività: "
MSG_TS_IM         = "Vuoi importare le attività da Calcurse? (y/n)"
MSG_TS_TW         = "Vuoi importare le attività da Taskwarrior? (y/n)"
MSG_TS_NOTHING    = "Nulla di pianificato..."
MSG_TS_PRIVACY    = "Attiva la privacy dell'attività con numero: "
MSG_TS_DEAD_ADD   = "Aggiungi una scadenza per l'attività con numero: "
MSG_TS_DEAD_DEL   = "Rimuovi la scadenza per l'attività con numero: "
MSG_TS_DEAD_DATE  = "Aggiungi una scandeza per il (AAAA/MM/GG): "
MSG_WEATHER       = "Caricamento del meteo..."

CALENDAR_HINT     = "Barra spaziatrice · Passa al diario   a · Aggiungi un evento  n/p · Cambia mese   ? · Mostra tutte le scorciatoie"
CALENDAR_HINT_D   = "Barra spaziatrice · Passa al diario   a · Aggiungi un evento  n/p · Cambia giorno  ? · Mostra tutte le scorciatoie"
JOURNAL_HINT      = "Barra spaziatrice · Passa al calendario   a · Add task   v · Finito   i · Importante   ? · Mostra tutte le scorciatoie"

DAYS = ["LUNEDÌ", "MARTEDÌ", "MERCOLEDÌ", "GIOVEDÌ", "VENERDì", "SABATO", "DOMENICA"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["GENNAIO", "FEBBRAIO", "MARZO", "APRILE", "MAGGIO", "GIUGNO", "LUGLIO", "AGOSTO", "SETTEMBRE", "OTTOBRE", "NOVEMBRE", "DICEMBRE"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""French translations of the program interface"""

MSG_WELCOME_1 = "Bienvenue à Calcure"
MSG_WELCOME_2 = "Le calendrier et le gestionnaire de tâches pour votre terminal!"
MSG_WELCOME_3 = "Les fichiers de configuration et de données ont été créés à:"
MSG_WELCOME_4 = "Pour obtenir de l'aide, des contributions et des informations supplémentaires, visitez:"
MSG_WELCOME_5 = "Presse ? pour voir les combinaisons de touches ou toute autre touche pour continuer."

TITLE_KEYS_GENERAL = "GÉNÉRALE"
TITLE_KEYS_CALENDAR = "CALENDRIER"
TITLE_KEYS_JOURNAL = "JOURNAL"

KEYS_GENERAL = {
        " Space ": "Basculer entre calendrier et journal",
        "   /   ": "Basculer l'écran partagé",
        "   *   ": "Basculer la confidentialité globale",
        "   ?   ": "Basculer cette aide",
        "   q   ": "Quitter",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Ajouter un événement (récurrent)",
        "  n,🠒  ": "Le mois (jour) prochain",
        "  p,🠐  ": "mois (jour) précédent",
        "  d,x  ": "Supprimer un événement",
        "  e,c  ": "Modifier un événement",
        "   m   ": "Déplacer l'événement",
        "   v   ": "Basculer la vue quotidienne/mensuelle",
        "   g   ": "Aller à un certain jour",
        "   h   ": "Désactiver l'événement en haute priorité",
        "   l   ": "Basculer l'événement en priorité basse",
        "   .   ": "Activer la confidentialité des événements",
        "   C   ": "Importer des événements depuis calcurse",
        "   G   ": "Revenir au mois (jour) en cours",
        }

KEYS_TODO = {
        "  a(A) ": "Ajouter une nouvelle (sous-)tâche",
        "  h(H) ": "Bascule une (toutes) des tâches en haute priorité",
        "  l(L) ": "Désactiver une (toutes) les tâches en priorité basse",
        "  v(V) ": "Bascule une (toutes) les tâches comme terminée",
        "  u(U) ": "Démarquer une (toutes) les tâches",
        "  d(D) ": "Supprimer une (toutes) les tâches (avec toutes les sous-tâches)",
        "  t(T) ": "Démarrer/mettre en pause (supprimer) le minuteur pour une tâche",
        "  e,c  ": "Modifier une tâche",
        "   s   ": "Basculer entre tâche et sous-tâche",
        "   .   ": "Basculer la confidentialité des tâches",
        "  f(F) ": "Modifier (supprimer) l'échéance de la tâche",
        "   m   ": "Déplacer une tâche",
        "  C(W) ": "Importer des tâches de calcurse (taskwarrior)",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "Les touches de Vim (j, k, ZZ, ZQ) fonctionnent également!"
MSG_INFO          = "Pour plus d'informations, visitez:"
MSG_SITE          = "https://github.com/anufrievroman/calcure"
MSG_EXIT          = "Vraiment quitter ? (y/n) "

MSG_EVENT_HIGH    = "Marquer comme numéro d'événement hautement prioritaire: "
MSG_EVENT_LOW     = "Marquer comme numéro d'événement basse priorité: "
MSG_EVENT_RESET   = "Réinitialiser le statut pour l'événement numéro : "
MSG_EVENT_DEL     = "Supprimer le numéro d'événement: "
MSG_EVENT_REN     = "Renommer le numéro d'événement: "
MSG_NEW_TITLE     = "Entrez un nouveau titre: "
MSG_EVENT_MOVE    = "Déplacer le numéro d'événement: "
MSG_EVENT_MOVE_TO = "Déplacer l'événement vers: "
MSG_EVENT_DATE    = "Entrez la date: "
MSG_EVENT_TITLE   = "Entrez le titre: "
MSG_EVENT_REP     = "Combien de répétitions de l'événement: "
MSG_EVENT_FR      = "Répéter l'événement tous les jours (d), semaines (w), mois (m) ou années (y) ?"
MSG_EVENT_IMP     = "Importer des événements depuis Calcurse ? (y/n)"
MSG_EVENT_PRIVACY = "Basculer la confidentialité du numéro d'événement: "
MSG_TM_ADD        = "Ajouter/mettre en pause le temporisateur pour la tâche numéro: "
MSG_TM_RESET      = "Supprimer le temporisateur pour la tâche numéro: "
MSG_TS_HIGH       = "Marquer comme haute priorité le numéro de tâche: "
MSG_TS_LOW        = "Marquer comme basse priorité le numéro de tâche: "
MSG_TS_RES        = "Réinitialiser l'état de la tâche numéro: "
MSG_TS_DONE       = "Marquer comme terminé le numéro de tâche: "
MSG_TS_DEL        = "Supprimer le numéro de tâche: "
MSG_TS_DEL_ALL    = "Vraiment supprimer toutes les tâches ? (y/n)"
MSG_TS_MOVE       = "Déplacer la tâche du numéro: "
MSG_TS_MOVE_TO    = "Déplacer la tâche vers le numéro: "
MSG_TS_EDIT       = "Modifier le numéro de tâche: "
MSG_TS_TOG        = "Basculer le numéro de sous-tâche: "
MSG_TS_SUB        = "Ajouter une sous-tâche pour la tâche numéro: "
MSG_TS_TITLE      = "Entrez la sous-tâche: "
MSG_TS_IM         = "Importer des tâches depuis Calcurse ? (y/n)"
MSG_TS_TW         = "Importer des tâches depuis Taskwarrior ? (y/n)"
MSG_TS_NOTHING    = "Rien de prévu..."
MSG_TS_PRIVACY    = "Basculer la confidentialité du numéro de tâche: "
MSG_TS_DEAD_ADD   = "Ajouter un délai pour la tâche numéro: "
MSG_TS_DEAD_DEL   = "Supprimer l'échéance de la tâche numéro: "
MSG_TS_DEAD_DATE  = "Ajouter une date limite le (AAAA/MM/JJ): "
MSG_WEATHER       = "La météo se charge..."

CALENDAR_HINT     = "Espace · Passer au journal  a · Ajouter un événement  n/p · Changer de mois  ? · Aider"
CALENDAR_HINT_D   = "Espace · Passer au journal  a · Ajouter un événement  n/p · Changer de jour  ? · All keybindings"
JOURNAL_HINT      = "Espace · Passer au calendrier  a · Ajouter une tâche  v · Terminé  i · Important  ? · All keybindings"

DAYS = ["LUNDI", "MARDI", "MERCREDI", "JEUDI", "VENDREDI", "SAMEDI", "DIMANCHE"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["JANVIER", "FEVRIER", "MARS", "AVRIL", "MAI", "JUIN", "JUILLET", "AOUT", "SEPTEMBRE", "OCTOBRE", "NOVEMBRE", "DECEMBRE"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""Module that controls import and export of the user data"""

import configparser
import pathlib
import csv
import os
import datetime

from ics import Calendar as IcsCalendar

from calcure.data import *
from calcure.helpers import convert_to_persian_date, convert_to_gregorian_date


def clean_ics_file(filename):
    """Remove multiple PRODID lines from ics file"""
    original_file = filename
    clean_file = filename + ".clean"
    previous_line = ""
    with open(original_file, 'r') as input_file, open(clean_file, 'w') as output_file:
        for line in input_file:
            # If there is more than one PRODID line, skip them:
            if not ("PRODID:" in line and "PRODID:" in previous_line):
                output_file.write(line)
            previous_line = line


class FileRepository:
    """Load and save events and tasks to files"""

    def __init__(self, cf):
        self.user_tasks = Tasks()
        self.user_events = Events()
        self.holidays = Events()
        self.birthdays = Birthdays()
        self.user_ics_tasks = Tasks()
        self.user_ics_events = Events()
        self.abook_file = str(pathlib.Path.home())+"/.abook/addressbook"
        self.tasks_file = cf.TASKS_FILE
        self.events_file = cf.EVENTS_FILE
        self.country = cf.HOLIDAY_COUNTRY
        self.use_persian_calendar = cf.USE_PERSIAN_CALENDAR
        self.ics_event_files = cf.ICS_EVENT_FILES
        self.ics_task_files = cf.ICS_TASK_FILES

    @property
    def is_task_format_old(self):
        """Check if the database format is old"""
        with open(self.tasks_file, "r", encoding="utf-8") as f:
            text = f.read()
        return text[0] == '"'

    def read_or_create_file(self, file):
        """Read user's csv file or create new one if it does not exist"""

        # Try to read the file line by line:
        try:
            with open(file, "r", encoding="utf-8") as f:
                read_lines = csv.reader(f, delimiter = ',')
                return list(read_lines)

        # Create file if it does not exist:
        except IOError:
            try:
                with open(file, "w+", encoding="utf-8") as f:
                    pass
                return []
            # Pass if there was a problem with file system:
            except (FileNotFoundError, NameError):
                return []

    def load_tasks_from_csv(self):
        """Reads from user's file or create new one if it does not exist"""
        lines = self.read_or_create_file(self.tasks_file)

        for index, row in enumerate(lines):
            task_id = index

            # Read task dates:
            if self.is_task_format_old:
                shift = 0
                year = 0
                month = 0
                day = 0
            else:
                shift = 3
                year = int(row[0])
                month = int(row[1])
                day = int(row[2])

            # Convert to persian date if needed and if it is not zero date:
            if self.use_persian_calendar and year != 0:
                year, month, day = convert_to_persian_date(year, month, day)

            # Read task name and statuses:
            if row[0 + shift][0] == '.':
                name = row[0 + shift][1:]
                is_private = True
            else:
                name = row[0 + shift]
                is_private = False
            status = Status[row[1 + shift].upper()]
            stamps = row[(2 + shift):] if len(row) > 2 else []
            timer = Timer(stamps)

            # Add task:
            new_task = Task(task_id, name, status, timer, is_private, year, month, day)
            self.user_tasks.add_item(new_task)
        return self.user_tasks

    def load_events_from_csv(self):
        """Reads from user's csv file or create it if it does not exist"""
        lines = self.read_or_create_file(self.events_file)
        for index, row in enumerate(lines):
            event_id = index
            year = int(row[1])
            month = int(row[2])
            day = int(row[3])
            if row[4][0] == '.':
                name = row[4][1:]
                is_private = True
            else:
                name = row[4]
                is_private = False

            # Account for old versions of the datafile:
            if len(row) > 5:
                repetition = int(row[5])
                if row[6] == 'd':
                    frequency = Frequency.DAILY
                elif row[6] == 'w':
                    frequency = Frequency.WEEKLY
                elif row[6] == 'm':
                    frequency = Frequency.MONTHLY
                elif row[6] == 'y':
                    frequency = Frequency.YEARLY
                else:
                    try:
                        frequency = Frequency[row[6].upper()]
                    except (ValueError, KeyError):
                        frequency = Frequency.ONCE
            else:
                repetition = '1'
                frequency = Frequency.ONCE
            if len(row) > 7:
                status = Status[row[7].upper()]
            else:
                status = Status.NORMAL

            # Convert to persian date if needed:
            if self.use_persian_calendar:
                year, month, day = convert_to_persian_date(year, month, day)

            # Add event:
            new_event = UserEvent(event_id, year, month, day, name, repetition, frequency, status, is_private)
            self.user_events.add_item(new_event)
        return self.user_events

    def save_tasks_to_csv(self):
        """Rewrite the data file with changed tasks"""
        original_file = self.tasks_file
        dummy_file = self.tasks_file + '.bak'
        with open(dummy_file, "w", encoding="utf-8") as f:
            for task in self.user_tasks.items:
                dot = "."

                # If persian calendar was used, we convert event back to Gregorian for storage:
                if self.use_persian_calendar and task.year != 0:
                    year, month, day = convert_to_gregorian_date(task.year, task.month, task.day)
                else:
                    year, month, day = task.year, task.month, task.day

                f.write(f'{year},{month},{day},"{dot*task.privacy}{task.name}",{task.status.name.lower()}')
                for stamp in task.timer.stamps:
                    f.write(f',{str(stamp)}')
                f.write("\n")
        os.remove(original_file)
        os.rename(dummy_file, original_file)
        self.user_tasks.changed = False

    def save_events_to_csv(self):
        """Rewrite the data file with changed events"""
        original_file = self.events_file
        dummy_file = self.events_file + '.bak'
        with open(dummy_file, "w", encoding="utf-8") as file:
            for ev in self.user_events.items:

                # If persian calendar was used, we convert event back to Gregorian for storage:
                if self.use_persian_calendar:
                    year, month, day = convert_to_gregorian_date(ev.year, ev.month, ev.day)
                else:
                    year, month, day = ev.year, ev. month, ev.day

                name = f'{"."*ev.privacy}{ev.name}'
                file.write(f'{ev.item_id},{year},{month},{day},"{name}",{ev.repetition},{ev.frequency.name.lower()},{ev.status.name.lower()}\n')
        os.remove(original_file)
        os.rename(dummy_file, original_file)
        self.user_events.changed = False

    def load_holidays(self):
        """Load list of holidays in this country around this year"""
        try:
            import holidays as hl
            year = datetime.date.today().year
            holiday_events = eval("hl."+self.country+"(years=[year-2, year-1, year, year+1, year+2, year+3, year+4])")
            for date, name in holiday_events.items():

                # Convert to persian date if needed:
                if self.use_persian_calendar:
                    year, month, day = convert_to_persian_date(date.year, date.month, date.day)
                else:
                    year, month, day = date.year, date.month, date.day

                # Add holiday:
                holiday = Event(year, month, day, name)
                self.holidays.add_item(holiday)
        except (ModuleNotFoundError, SyntaxError, AttributeError):
            pass
        return self.holidays

    def load_birthdays_from_abook(self):
        """Loading birthdays from abook contacts"""

        # Quit if file does not exists:
        if not os.path.exists(self.abook_file):
            return self.birthdays

        abook = configparser.ConfigParser()
        abook.read(self.abook_file)
        for each_contact in abook.sections():
            for key, _ in abook.items(each_contact):
                if key in ["birthday", "anniversary"]:
                    month = int(abook[each_contact][key][-5:-3])
                    day = int(abook[each_contact][key][-2:])
                    name = abook[each_contact]["name"]

                    # Convert to persian date if needed:
                    if self.use_persian_calendar:
                        _, month, day = convert_to_persian_date(1000, month, day)

                    # Add birthday:
                    birthday = Event(1, month, day, name)
                    self.birthdays.add_item(birthday)
        return self.birthdays

    def load_tasks_from_ics(self):
        """Load tasks from each of the ics files"""

        # Quit if the file is not specified:
        if self.ics_task_files is None:
            return self.user_ics_tasks

        for filename in self.ics_task_files:

            # Quit if file does not exists:
            if not os.path.exists(filename):
                return self.user_ics_tasks

            # Clean and read the file, then remove tmp file:
            clean_ics_file(filename)
            with open(filename + ".clean", 'r', encoding="utf-8") as file:
                ics_text = file.read()
            os.remove(filename + ".clean")

            cal = IcsCalendar(ics_text)
            for task in cal.todos:
                if task.status != "CANCELLED":
                    task_id = self.user_ics_tasks.generate_id()

                    # Assign status from priority:
                    status = Status.NORMAL
                    if task.priority is not None:
                        if task.priority > 5:
                            status = Status.UNIMPORTANT
                        if task.priority < 5:
                            status = Status.IMPORTANT

                    # Correct according to status:
                    if task.status == "COMPLETED":
                        status = Status.DONE

                    name = task.name

                    # Try reading task due date:
                    try:
                        year = task.due.year
                        month = task.due.month
                        day = task.due.day
                    except AttributeError:
                        year, month, day = 0, 0, 0

                    timer = Timer([])
                    is_private = False

                    # Add task:
                    new_task = Task(task_id, name, status, timer, is_private, year, month, day)
                    self.user_ics_tasks.add_item(new_task)

        return self.user_ics_tasks

    def load_events_from_ics(self):
        """Load events from each of the ics files"""

        # Quit if the file is not specified:
        if self.ics_event_files is None:
            return self.user_ics_events

        for filename in self.ics_event_files:

            # Quit if file does not exists:
            if not os.path.exists(filename):
                return self.user_ics_events

            # Clean and read the file, then remove tmp file:
            clean_ics_file(filename)
            with open(filename + ".clean", 'r', encoding="utf-8") as file:
                ics_text = file.read()
            os.remove(filename + ".clean")
            cal = IcsCalendar(ics_text)
            for index, event in enumerate(cal.events):

                # Default parameters:
                event_id = index
                repetition = '1'
                frequency = Frequency.ONCE
                status = Status.NORMAL
                is_private = False

                # Parameters of the event from ics if they exist:
                name = event.name if event.name is not None else ""
                all_day = event.all_day if event.all_day is not None else True
                year = event.begin.year if event.begin else 0
                month = event.begin.month if event.begin else 1
                day = event.begin.day if event.begin else 1

                # Add start time to name of non-all-day events:
                if not all_day:
                    hour = event.begin.hour if event.begin else 0
                    minute = event.begin.minute if event.begin else 0
                    name = f"{hour:0=2}:{minute:0=2} {name}"

                # Convert to persian date if needed:
                if self.use_persian_calendar:
                    year, month, day = convert_to_persian_date(year, month, day)

                # Add event:
                new_event = UserEvent(event_id, year, month, day, name, repetition, frequency, status, is_private)
                self.user_ics_events.add_item(new_event)

        return self.user_ics_events



"""Module that provides few converter functions"""

import curses
import jdatetime

from calcure.data import Color


def convert_to_persian_date(year, month, day):
    """Convert date from Gregorian to Persian calendar"""
    persian_date =  jdatetime.date.fromgregorian(day=day, month=month, year=year)
    day = persian_date.day
    month = persian_date.month
    year = persian_date.year
    return year, month, day


def convert_to_gregorian_date(year, month, day):
    """Convert date from Persian to Gregorian calendar"""
    gregorian_date = jdatetime.date(year, month, day).togregorian()
    day = gregorian_date.day
    month = gregorian_date.month
    year = gregorian_date.year
    return year, month, day


def initialize_colors(cf):
    """Define all the color pairs"""
    curses.start_color()
    curses.use_default_colors()
    curses.init_pair(Color.DAY_NAMES.value, cf.COLOR_DAY_NAMES, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.WEEKENDS.value, cf.COLOR_WEEKENDS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.HINTS.value, cf.COLOR_HINTS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.TODAY.value, cf.COLOR_TODAY, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.DAYS.value, cf.COLOR_DAYS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.WEEKEND_NAMES.value, cf.COLOR_WEEKEND_NAMES, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.BIRTHDAYS.value, cf.COLOR_BIRTHDAYS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.PROMPTS.value, cf.COLOR_PROMPTS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.CONFIRMATIONS.value, cf.COLOR_CONFIRMATIONS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.TITLE.value, cf.COLOR_TITLE, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.TODO.value, cf.COLOR_TODO, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.DONE.value, cf.COLOR_DONE, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.IMPORTANT.value, cf.COLOR_IMPORTANT, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.TIMER.value, cf.COLOR_TIMER, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.TIMER_PAUSED.value, cf.COLOR_TIMER_PAUSED, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.HOLIDAYS.value, cf.COLOR_HOLIDAYS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.EVENTS.value, cf.COLOR_EVENTS, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.TIME.value, cf.COLOR_TIME, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.WEATHER.value, cf.COLOR_WEATHER, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.UNIMPORTANT.value, cf.COLOR_UNIMPORTANT, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.CALENDAR_HEADER.value, cf.COLOR_CALENDAR_HEADER, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.ACTIVE_PANE.value, cf.COLOR_ACTIVE_PANE, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.SEPARATOR.value, cf.COLOR_SEPARATOR, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.EMPTY.value, cf.COLOR_BACKGROUND, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.CALENDAR_BOARDER.value, cf.COLOR_CALENDAR_BOARDER, cf.COLOR_BACKGROUND)
    curses.init_pair(Color.DEADLINES.value, cf.COLOR_DEADLINES, cf.COLOR_BACKGROUND)

    if not cf.MINIMAL_WEEKEND_INDICATOR:
        curses.init_pair(Color.WEEKENDS.value, curses.COLOR_BLACK, cf.COLOR_WEEKENDS)
    if not cf.MINIMAL_TODAY_INDICATOR:
        curses.init_pair(Color.TODAY.value, curses.COLOR_BLACK, cf.COLOR_TODAY)
    if not cf.MINIMAL_DAYS_INDICATOR:
        curses.init_pair(Color.DAYS.value, curses.COLOR_BLACK, cf.COLOR_DAYS)

